import { Component, OnInit, AfterViewInit, AfterContentInit, ViewChild, OnDestroy } from '@angular/core';
import { Router } from '@angular/router';

import { Store, select } from '@ngrx/store';

import { RPCClientError } from '@loafer/ng-rpc/protocol';
import { Domain } from '@overflow/commons-typescript/model/domain';
import { AuthSelector } from 'packages/member/store';

import { Probe, ProbeHost } from '@overflow/commons-typescript/model/probe';
import * as ListStore from '../../store/probe-host-list';
import { ProbeHostListSelector } from '../../store';
import { Subscription } from 'rxjs/Subscription';

@Component({
  selector: 'of-probe-list',
  templateUrl: './list.component.html',
})
export class ListComponent implements OnInit, AfterContentInit, OnDestroy {
  probeHostsSubscription$: Subscription;
  probeHosts$ = this.store.pipe(select(ProbeHostListSelector.select('probeHosts')));
  probeHosts: ProbeHost[];

  constructor(
    private router: Router,
    private store: Store<ListStore.State>
  ) {
  }

  ngOnInit() {
    this.probeHostsSubscription$ = this.probeHosts$.subscribe(
      (probeHosts: ProbeHost[]) => {
        console.log(probeHosts);
        this.probeHosts = probeHosts;
      },
      (error: RPCClientError) => {
        console.log(error.response.message);
      }
    );
  }

  ngAfterContentInit() {
    this.store.select(AuthSelector.select('domain')).subscribe(
      (domain: Domain) => {
        this.store.dispatch(new ListStore.ReadAllByDomain(domain));
      },
      (error) => {
        console.log(error);
      }
    );
  }

  ngOnDestroy() {
    if (this.probeHostsSubscription$) {
      this.probeHostsSubscription$.unsubscribe();
    }
  }

  onRowSelect(event) {
    this.router.navigate(['probe', event.data.probe.id, 'info']);
  }

  getUptime(probe: Probe) {
    // if (probe.connectDate === null || probe.connectDate ==== undefined) {
    //   return 'Not Connected.';
    // }
    // const currentDate = new Date();

    return 'Uptime구해와';
  }
}
